@@ -7,7 +7,7 @@
 *
 *	This file part of:	PSFEx
 *
-*	Copyright:		(C) 2008-2012 Emmanuel Bertin -- IAP/CNRS/UPMC
+*	Copyright:		(C) 2008-2018 IAP/CNRS/SorbonneU
 *
 *	License:		GNU General Public License
 *
@@ -22,7 +22,7 @@
 *	You should have received a copy of the GNU General Public License
 *	along with PSFEx.  If not, see <http://www.gnu.org/licenses/>.
 *
-*	Last modified:		16/04/2012
+*	Last modified:		06/12/2018
 *
 *%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
@@ -73,6 +73,7 @@ int	plotnum[CPLOT_NTYPES];
 int	plotdev[CPLOT_NTYPES];
 char	plotfilename[MAXCHAR];
 int	plotaaflag;
+double	plotaspect;
 
 /****** cplot_check ***********************************************************
 PROTO	int cplot_check(cplotenum cplottype)
@@ -99,24 +100,24 @@ int	 cplot_check(cplotenum cplottype)
 
 
 /****** cplot_init ***********************************************************
-PROTO	int cplot_init(char *name, int nx, int ny, cplotenum cplottype)
+PROTO	int cplot_init(char *name, cplotenum cplottype)
 PURPOSE	Initialize a check plot.
 INPUT	Input name,
-	Number of plots along the x axis,
-	number of plots along the y axis,
 	plot type,
 	device number.
 OUTPUT	RETURN_OK if everything went fine, RETURN_ERROR otherwise.
 NOTES	.
 AUTHOR	E. Bertin (IAP)
-VERSION	10/09/2009
+VERSION	19/10/2016
  ***/
-int	cplot_init(char *name, int nx, int ny, cplotenum cplottype)
+int	cplot_init(char *name, cplotenum cplottype)
   {
+   PLFLT	p_xp,p_yp;
+   PLINT	p_xleng,p_yleng, p_xoff,p_yoff;
    char		**devmenu, **devname;
    char		str[MAXCHAR],
 		*pstr;
-   int		j, num, cval, dev, argc, ndev, gdflag;
+   int		j, num, cval, dev, argc, ndev, gdflag, ret;
 
 /* Check that plot was requested */
   cval = cplot_check(cplottype);
@@ -133,7 +134,7 @@ int	cplot_init(char *name, int nx, int ny, cplotenum cplottype)
 	prefs.cplot_res[0]? prefs.cplot_res[0] : CPLOT_DEFRESX,
 	prefs.cplot_res[1]? prefs.cplot_res[1] : CPLOT_DEFRESY,
 	plotfilename, plotfilename);
-    system(str);
+    ret = system(str);
     }
 
   if (dev>=prefs.ncplot_device)
@@ -155,7 +156,6 @@ int	cplot_init(char *name, int nx, int ny, cplotenum cplottype)
 			cplot_device[j].extension);
     plsfnam(plotfilename);		/* file name */
     }
-  plssub(nx, ny);
   plsdev(cplot_device[j].devname);
 
   plotaaflag = 0;
@@ -212,6 +212,8 @@ int	cplot_init(char *name, int nx, int ny, cplotenum cplottype)
   plscol0(7, 128,128,128);	/* Force the midground colour to grey */
   plscol0(8, 64,0,0);		/* Force the brown colour to darken */
   plinit();
+  plgpage(&p_xp, &p_yp, &p_xleng, &p_yleng, &p_xoff, &p_yoff);
+  plotaspect = (double)p_yleng / (double)p_xleng;
 
   return RETURN_OK;
   }
@@ -405,7 +407,7 @@ INPUT	Pointer to the WCS projection structure,
 OUTPUT	RETURN_OK if everything went fine, RETURN_ERROR otherwise.
 NOTES	.
 AUTHOR	E. Bertin (IAP)
-VERSION	26/10/2005
+VERSION	06/12/2018
  ***/
 int	cplot_drawloccoordgrid(wcsstruct *wcs, double xmin, double xmax,
 				double ymin, double ymax)
@@ -527,7 +529,7 @@ int	cplot_drawloccoordgrid(wcsstruct *wcs, double xmin, double xmax,
 
 /* Draw meridians */
   plschr(0.0, 0.33);
-  plwid(0);
+  CPLOT_PLWID(0);
   pllsty(2);
   xmd = xmu = xdo = -0.5;
   ymd = ymu = ydo = -0.5;
@@ -546,7 +548,8 @@ int	cplot_drawloccoordgrid(wcsstruct *wcs, double xmin, double xmax,
 		&& (xm = (xd-xmin)/dx) > 0.0 && xm < 1.0
 		&& fabs(xm-xmd) > 0.1)
           {
-          plmtex("b", 2.0, (PLFLT)xm, 0.5, cplot_degtosexde(str,wcspos[0],
+          plmtex("b", 2.0, (PLFLT)xm, 0.5,
+		cplot_degtosexde(str,fmod_m90_p90(wcspos[0]),
 		alphastep));
           xmd = xm;
           }
@@ -554,7 +557,8 @@ int	cplot_drawloccoordgrid(wcsstruct *wcs, double xmin, double xmax,
 		&& (xm = (xd-xmin)/dx) > 0.0 && xm < 1.0
 		&& fabs(xm-xmu) > 0.1)
           {
-          plmtex("t", 1.5, (PLFLT)xm, 0.5, cplot_degtosexde(str,wcspos[0],
+          plmtex("t", 1.5, (PLFLT)xm, 0.5,
+		cplot_degtosexde(str,fmod_m90_p90(wcspos[0]),
 		alphastep));
           xmu = xm;
           }
@@ -663,7 +667,7 @@ INPUT	Pointer to the field.
 OUTPUT	RETURN_OK if everything went fine, RETURN_ERROR otherwise.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	16/04/2012
+VERSION	19/10/2016
  ***/
 int	cplot_fwhm(fieldstruct *field)
   {
@@ -681,7 +685,7 @@ int	cplot_fwhm(fieldstruct *field)
    int		naxisn[NAXIS],
 		i,j, e, n,n2,ncx,ncy,nt, nfwhm, naxis, nsnap2, flag;
 
-  if (cplot_init(field->rcatname, 1,1, CPLOT_FWHM) == RETURN_ERROR)
+  if (cplot_init(field->rcatname, CPLOT_FWHM) == RETURN_ERROR)
     {
     cplot_end(CPLOT_FWHM);
     return RETURN_OK;
@@ -709,14 +713,14 @@ int	cplot_fwhm(fieldstruct *field)
   ymin = 0.5;
   ymax = 100.5;
   lwid = plotaaflag? ((CPLOT_AAFAC+1)/2) : 1;
-  plwid(lwid);
+  CPLOT_PLWID(lwid);
   plfont(2);
   plcol0(15);
   plenv((PLFLT)xmin, (PLFLT)xmax, (PLFLT)ymin, (PLFLT)ymax, 1, -1);
   sprintf(str, "#uField %.24s: FWHM map", field->rtcatname);
   plschr(0.0, 1.0);
   pllab("","", str);
-  plwid(0);
+  CPLOT_PLWID(0);
   plcol0(7);
   cplot_drawloccoordgrid(wcsout, xmin, xmax, ymin, ymax);
 
@@ -823,14 +827,14 @@ int	cplot_fwhm(fieldstruct *field)
 
       wcsptr[0] = wcs;
       wcsptr[1] = wcsout;
-      plshades(fwhm, nsnap2, nsnap2, NULL,
+      plshades((const PLFLT * const*)fwhm, nsnap2, nsnap2, NULL,
 	     xstep/2.0+0.5, wcs->naxisn[0]-xstep/2.0+0.5,
              ystep/2.0+0.5, wcs->naxisn[1]-ystep/2.0+0.5,
 	     clevel, CPLOT_NSHADES, 1, 0, 0, plfill, 0, distort_map, wcsptr);
       plFree2dGrid(fwhm, nsnap2, nsnap2);
       }
     plcol0(7);
-    plwid(lwid);
+    CPLOT_PLWID(lwid);
     cplot_drawbounds(wcs, wcsout);
     }
 
@@ -843,14 +847,14 @@ int	cplot_fwhm(fieldstruct *field)
   if (wcs->lng == wcs->lat)
     {
     plwind(0.0,1.0,fwhmmin,fwhmmax);
-    plshades(fwhm, 2, CPLOT_NSHADES, NULL, 0.0, 1.0,
+    plshades((const PLFLT * const*)fwhm, 2, CPLOT_NSHADES, NULL, 0.0, 1.0,
 	   fwhmmin,fwhmmax, clevel,
 	   CPLOT_NSHADES, 1, 0, 0, plfill, 1, NULL, NULL);
     }
   else
     {
     plwind(0.0,1.0,fwhmmin*DEG/ARCSEC,fwhmmax*DEG/ARCSEC);
-    plshades(fwhm, 2, CPLOT_NSHADES, NULL, 0.0, 1.0,
+    plshades((const PLFLT * const*)fwhm, 2, CPLOT_NSHADES, NULL, 0.0, 1.0,
 	   fwhmmin*DEG/ARCSEC,fwhmmax*DEG/ARCSEC, clevel,
 	   CPLOT_NSHADES, 1, 0, 0, plfill, 1, NULL, NULL);
     }
@@ -895,7 +899,7 @@ INPUT	Pointer to the field.
 OUTPUT	RETURN_OK if everything went fine, RETURN_ERROR otherwise.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	16/04/2012
+VERSION	19/10/2016
  ***/
 int	cplot_ellipticity(fieldstruct *field)
   {
@@ -913,7 +917,7 @@ int	cplot_ellipticity(fieldstruct *field)
    int		naxisn[NAXIS],
 		i,j, e, n,n2, ncx,ncy,nt, nellip, naxis, nsnap2, flag;
 
-  if (cplot_init(field->rcatname, 1,1, CPLOT_ELLIPTICITY) == RETURN_ERROR)
+  if (cplot_init(field->rcatname, CPLOT_ELLIPTICITY) == RETURN_ERROR)
     {
     cplot_end(CPLOT_ELLIPTICITY);
     return RETURN_OK;
@@ -941,14 +945,14 @@ int	cplot_ellipticity(fieldstruct *field)
   ymin = 0.5;
   ymax = 100.5;
   lwid = plotaaflag? ((CPLOT_AAFAC+1)/2) : 1;
-  plwid(lwid);
+  CPLOT_PLWID(lwid);
   plfont(2);
   plcol0(15);
   plenv((PLFLT)xmin, (PLFLT)xmax, (PLFLT)ymin, (PLFLT)ymax, 1, -1);
   sprintf(str, "#uField %.24s: ellipticity map", field->rtcatname);
   plschr(0.0, 1.0);
   pllab("","", str);
-  plwid(0);
+  CPLOT_PLWID(0);
   plcol0(7);
   cplot_drawloccoordgrid(wcsout, xmin, xmax, ymin, ymax);
 
@@ -1055,14 +1059,14 @@ int	cplot_ellipticity(fieldstruct *field)
 
       wcsptr[0] = wcs;
       wcsptr[1] = wcsout;
-      plshades(ellip, nsnap2, nsnap2, NULL,
+      plshades((const PLFLT * const*)ellip, nsnap2, nsnap2, NULL,
 	     xstep/2.0+0.5, wcs->naxisn[0]-xstep/2.0+0.5,
              ystep/2.0+0.5, wcs->naxisn[1]-ystep/2.0+0.5,
 	     clevel, CPLOT_NSHADES, 1, 0, 0, plfill, 0, distort_map, wcsptr);
       plFree2dGrid(ellip, nsnap2, nsnap2);
       }
     plcol0(7);
-    plwid(lwid);
+    CPLOT_PLWID(lwid);
     cplot_drawbounds(wcs, wcsout);
     }
 
@@ -1073,7 +1077,7 @@ int	cplot_ellipticity(fieldstruct *field)
 
   plvpor(0.91,0.935,0.115,0.885);
   plwind(0.0,1.0,ellipmin,ellipmax);
-  plshades(ellip, 2, CPLOT_NSHADES, NULL, 0.0, 1.0,
+  plshades((const PLFLT * const*)ellip, 2, CPLOT_NSHADES, NULL, 0.0, 1.0,
 	   ellipmin,ellipmax, clevel,
 	   CPLOT_NSHADES, 1, 0, 0, plfill, 1, NULL, NULL);
   plcol0(15);
@@ -1113,7 +1117,7 @@ INPUT	Pointer to the field.
 OUTPUT	RETURN_OK if everything went fine, RETURN_ERROR otherwise.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	16/04/2012
+VERSION	19/10/2016
  ***/
 int	cplot_moffatresi(fieldstruct *field)
   {
@@ -1131,7 +1135,7 @@ int	cplot_moffatresi(fieldstruct *field)
    int		naxisn[NAXIS],
 		i,j, e, n,n2,ncx,ncy,nt, nresi, naxis, nsnap2, flag;
 
-  if (cplot_init(field->rcatname, 1,1, CPLOT_MOFFATRESI) == RETURN_ERROR)
+  if (cplot_init(field->rcatname, CPLOT_MOFFATRESI) == RETURN_ERROR)
     {
     cplot_end(CPLOT_MOFFATRESI);
     return RETURN_OK;
@@ -1159,14 +1163,14 @@ int	cplot_moffatresi(fieldstruct *field)
   ymin = 0.5;
   ymax = 100.5;
   lwid = plotaaflag? ((CPLOT_AAFAC+1)/2) : 1;
-  plwid(lwid);
+  CPLOT_PLWID(lwid);
   plfont(2);
   plcol0(15);
   plenv((PLFLT)xmin, (PLFLT)xmax, (PLFLT)ymin, (PLFLT)ymax, 1, -1);
   sprintf(str, "#uField %.24s: map of Moffat fit residuals", field->rtcatname);
   plschr(0.0, 1.0);
   pllab("","", str);
-  plwid(0);
+  CPLOT_PLWID(0);
   plcol0(7);
   cplot_drawloccoordgrid(wcsout, xmin, xmax, ymin, ymax);
 
@@ -1271,14 +1275,14 @@ int	cplot_moffatresi(fieldstruct *field)
 
       wcsptr[0] = wcs;
       wcsptr[1] = wcsout;
-      plshades(resi, nsnap2, nsnap2, NULL,
+      plshades((const PLFLT * const*)resi, nsnap2, nsnap2, NULL,
 	     xstep/2.0+0.5, wcs->naxisn[0]-xstep/2.0+0.5,
              ystep/2.0+0.5, wcs->naxisn[1]-ystep/2.0+0.5,
 	     clevel, CPLOT_NSHADES, 1, 0, 0, plfill, 0, distort_map, wcsptr);
       plFree2dGrid(resi, nsnap2, nsnap2);
       }
     plcol0(7);
-    plwid(lwid);
+    CPLOT_PLWID(lwid);
     cplot_drawbounds(wcs, wcsout);
     }
 
@@ -1289,7 +1293,7 @@ int	cplot_moffatresi(fieldstruct *field)
 
   plvpor(0.91,0.935,0.115,0.885);
   plwind(0.0,1.0,resimin,resimax);
-  plshades(resi, 2, CPLOT_NSHADES, NULL, 0.0, 1.0,
+  plshades((const PLFLT * const*)resi, 2, CPLOT_NSHADES, NULL, 0.0, 1.0,
 	   resimin,resimax, clevel,
 	   CPLOT_NSHADES, 1, 0, 0, plfill, 1, NULL, NULL);
   plcol0(15);
@@ -1332,7 +1336,7 @@ INPUT	Pointer to the field.
 OUTPUT	RETURN_OK if everything went fine, RETURN_ERROR otherwise.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	16/04/2012
+VERSION	19/10/2016
  ***/
 int	cplot_asymresi(fieldstruct *field)
   {
@@ -1350,7 +1354,7 @@ int	cplot_asymresi(fieldstruct *field)
    int		naxisn[NAXIS],
 		i,j, e, n,n2,ncx,ncy,nt, nresi, naxis, nsnap2, flag;
 
-  if (cplot_init(field->rcatname, 1,1, CPLOT_ASYMRESI) == RETURN_ERROR)
+  if (cplot_init(field->rcatname, CPLOT_ASYMRESI) == RETURN_ERROR)
     {
     cplot_end(CPLOT_ASYMRESI);
     return RETURN_OK;
@@ -1378,14 +1382,14 @@ int	cplot_asymresi(fieldstruct *field)
   ymin = 0.5;
   ymax = 100.5;
   lwid = plotaaflag? ((CPLOT_AAFAC+1)/2) : 1;
-  plwid(lwid);
+  CPLOT_PLWID(lwid);
   plfont(2);
   plcol0(15);
   plenv((PLFLT)xmin, (PLFLT)xmax, (PLFLT)ymin, (PLFLT)ymax, 1, -1);
   sprintf(str, "#uField %.24s: PSF asymmetry map", field->rtcatname);
   plschr(0.0, 1.0);
   pllab("","", str);
-  plwid(0);
+  CPLOT_PLWID(0);
   plcol0(7);
   cplot_drawloccoordgrid(wcsout, xmin, xmax, ymin, ymax);
 
@@ -1490,14 +1494,14 @@ int	cplot_asymresi(fieldstruct *field)
 
       wcsptr[0] = wcs;
       wcsptr[1] = wcsout;
-      plshades(resi, nsnap2, nsnap2, NULL,
+      plshades((const PLFLT * const*)resi, nsnap2, nsnap2, NULL,
 	     xstep/2.0+0.5, wcs->naxisn[0]-xstep/2.0+0.5,
              ystep/2.0+0.5, wcs->naxisn[1]-ystep/2.0+0.5,
 	     clevel, CPLOT_NSHADES, 1, 0, 0, plfill, 0, distort_map, wcsptr);
       plFree2dGrid(resi, nsnap2, nsnap2);
       }
     plcol0(7);
-    plwid(lwid);
+    CPLOT_PLWID(lwid);
     cplot_drawbounds(wcs, wcsout);
     }
 
@@ -1508,7 +1512,7 @@ int	cplot_asymresi(fieldstruct *field)
 
   plvpor(0.91,0.935,0.115,0.885);
   plwind(0.0,1.0,resimin,resimax);
-  plshades(resi, 2, CPLOT_NSHADES, NULL, 0.0, 1.0,
+  plshades((const PLFLT * const*)resi, 2, CPLOT_NSHADES, NULL, 0.0, 1.0,
 	   resimin,resimax, clevel,
 	   CPLOT_NSHADES, 1, 0, 0, plfill, 1, NULL, NULL);
   plcol0(15);
@@ -1546,7 +1550,7 @@ INPUT	Pointer to the field.
 OUTPUT	RETURN_OK if everything went fine, RETURN_ERROR otherwise.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	16/04/2012
+VERSION	19/10/2016
  ***/
 int	cplot_counts(fieldstruct *field)
   {
@@ -1564,7 +1568,7 @@ int	cplot_counts(fieldstruct *field)
 		**icount,
 		i,j, e, n,nt, naxis, nsnap,nsnap2, flag;
 
-  if (cplot_init(field->rcatname, 1,1, CPLOT_COUNTS) == RETURN_ERROR)
+  if (cplot_init(field->rcatname, CPLOT_COUNTS) == RETURN_ERROR)
     {
     cplot_end(CPLOT_COUNTS);
     return RETURN_OK;
@@ -1592,14 +1596,14 @@ int	cplot_counts(fieldstruct *field)
   ymin = 0.5;
   ymax = 100.5;
   lwid = plotaaflag? ((CPLOT_AAFAC+1)/2) : 1;
-  plwid(lwid);
+  CPLOT_PLWID(lwid);
   plfont(2);
   plcol0(15);
   plenv((PLFLT)xmin, (PLFLT)xmax, (PLFLT)ymin, (PLFLT)ymax, 1, -1);
   sprintf(str, "#uField %.24s: source count map", field->rtcatname);
   plschr(0.0, 1.0);
   pllab("","", str);
-  plwid(0);
+  CPLOT_PLWID(0);
   plcol0(7);
   cplot_drawloccoordgrid(wcsout, xmin, xmax, ymin, ymax);
 
@@ -1660,13 +1664,13 @@ int	cplot_counts(fieldstruct *field)
           count[i][j] = (PLFLT)icount[e][0];
     wcsptr[0] = wcs;
     wcsptr[1] = wcsout;
-    plshades(count, nsnap2, nsnap2, NULL,
+    plshades((const PLFLT * const*)count, nsnap2, nsnap2, NULL,
 	     xstep/2.0+0.5, wcs->naxisn[0]-xstep/2.0+0.5,
              ystep/2.0+0.5, wcs->naxisn[1]-ystep/2.0+0.5,
 	     clevel, CPLOT_NSHADES, 1, 0, 0, plfill, 0, distort_map, wcsptr);
     plFree2dGrid(count, nsnap2, nsnap2);
     plcol0(7);
-    plwid(lwid);
+    CPLOT_PLWID(lwid);
     cplot_drawbounds(wcs, wcsout);
     }
 
@@ -1677,8 +1681,8 @@ int	cplot_counts(fieldstruct *field)
 
   plvpor(0.91,0.935,0.115,0.885);
   plwind(0.0,1.0,cmin,cmax);
-  plshades(count, 2, CPLOT_NSHADES, NULL, 0.0, 1.0, cmin,cmax, clevel,
-	   CPLOT_NSHADES, 1, 0, 0, plfill, 1, NULL, NULL);
+  plshades((const PLFLT * const*)count, 2, CPLOT_NSHADES, NULL, 0.0, 1.0,
+	   cmin,cmax, clevel, CPLOT_NSHADES, 1, 0, 0, plfill, 1, NULL, NULL);
   plcol0(15);
   plschr(0.0, 0.5);
   plbox("bc", 0.0, 0, "bnstv", 0.0, 0);
@@ -1705,7 +1709,7 @@ INPUT	Pointer to the field.
 OUTPUT	RETURN_OK if everything went fine, RETURN_ERROR otherwise.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	16/04/2012
+VERSION	19/10/2016
  ***/
 int	cplot_countfrac(fieldstruct *field)
   {
@@ -1723,7 +1727,7 @@ int	cplot_countfrac(fieldstruct *field)
 		**iacount,**ilcount,
 		i,j, e, n,nt, naxis, nsnap,nsnap2, flag;
 
-  if (cplot_init(field->rcatname, 1,1, CPLOT_COUNTFRAC) == RETURN_ERROR)
+  if (cplot_init(field->rcatname, CPLOT_COUNTFRAC) == RETURN_ERROR)
     {
     cplot_end(CPLOT_COUNTFRAC);
     return RETURN_OK;
@@ -1751,14 +1755,14 @@ int	cplot_countfrac(fieldstruct *field)
   ymin = 0.5;
   ymax = 100.5;
   lwid = plotaaflag? ((CPLOT_AAFAC+1)/2) : 1;
-  plwid(lwid);
+  CPLOT_PLWID(lwid);
   plfont(2);
   plcol0(15);
   plenv((PLFLT)xmin, (PLFLT)xmax, (PLFLT)ymin, (PLFLT)ymax, 1, -1);
   sprintf(str, "#uField %.24s: source count fraction map", field->rtcatname);
   plschr(0.0, 1.0);
   pllab("","", str);
-  plwid(0);
+  CPLOT_PLWID(0);
   plcol0(7);
   cplot_drawloccoordgrid(wcsout, xmin, xmax, ymin, ymax);
 
@@ -1820,13 +1824,13 @@ int	cplot_countfrac(fieldstruct *field)
 
     wcsptr[0] = wcs;
     wcsptr[1] = wcsout;
-    plshades(count, nsnap2, nsnap2, NULL,
+    plshades((const PLFLT * const*)count, nsnap2, nsnap2, NULL,
 	     xstep/2.0+0.5, wcs->naxisn[0]-xstep/2.0+0.5,
              ystep/2.0+0.5, wcs->naxisn[1]-ystep/2.0+0.5,
 	     clevel, CPLOT_NSHADES, 1, 0, 0, plfill, 0, distort_map, wcsptr);
     plFree2dGrid(count, nsnap2, nsnap2);
     plcol0(7);
-    plwid(lwid);
+    CPLOT_PLWID(lwid);
     cplot_drawbounds(wcs, wcsout);
     }
 
@@ -1837,8 +1841,8 @@ int	cplot_countfrac(fieldstruct *field)
 
   plvpor(0.91,0.935,0.115,0.885);
   plwind(0.0,1.0,cmin,cmax);
-  plshades(count, 2, CPLOT_NSHADES, NULL, 0.0, 1.0, cmin,cmax, clevel,
-	   CPLOT_NSHADES, 1, 0, 0, plfill, 1, NULL, NULL);
+  plshades((const PLFLT * const*)count, 2, CPLOT_NSHADES, NULL, 0.0, 1.0,
+	   cmin,cmax, clevel, CPLOT_NSHADES, 1, 0, 0, plfill, 1, NULL, NULL);
   plcol0(15);
   plschr(0.0, 0.5);
   plbox("bc", 0.0, 0, "bnstv", 0.0, 0);
@@ -1864,7 +1868,7 @@ INPUT	Pointer to the field.
 OUTPUT	RETURN_OK if everything went fine, RETURN_ERROR otherwise.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	16/04/2012
+VERSION	19/10/2016
  ***/
 int	cplot_modchi2(fieldstruct *field)
   {
@@ -1883,7 +1887,7 @@ int	cplot_modchi2(fieldstruct *field)
 		**scount,
 		i,j, e, n,nt, naxis, nsnap,nsnap2, flag;
 
-  if (cplot_init(field->rcatname, 1,1, CPLOT_CHI2) == RETURN_ERROR)
+  if (cplot_init(field->rcatname, CPLOT_CHI2) == RETURN_ERROR)
     {
     cplot_end(CPLOT_CHI2);
     return RETURN_OK;
@@ -1911,14 +1915,14 @@ int	cplot_modchi2(fieldstruct *field)
   ymin = 0.5;
   ymax = 100.5;
   lwid = plotaaflag? ((CPLOT_AAFAC+1)/2) : 1;
-  plwid(lwid);
+  CPLOT_PLWID(lwid);
   plfont(2);
   plcol0(15);
   plenv((PLFLT)xmin, (PLFLT)xmax, (PLFLT)ymin, (PLFLT)ymax, 1, -1);
   sprintf(str, "#uField %.24s: #gx#u2#d/d.o.f. map", field->rtcatname);
   plschr(0.0, 1.0);
   pllab("","", str);
-  plwid(0);
+  CPLOT_PLWID(0);
   plcol0(7);
   cplot_drawloccoordgrid(wcsout, xmin, xmax, ymin, ymax);
 
@@ -1979,13 +1983,13 @@ int	cplot_modchi2(fieldstruct *field)
 
     wcsptr[0] = wcs;
     wcsptr[1] = wcsout;
-    plshades(count, nsnap2, nsnap2, NULL,
+    plshades((const PLFLT * const*)count, nsnap2, nsnap2, NULL,
 	     xstep/2.0+0.5, wcs->naxisn[0]-xstep/2.0+0.5,
              ystep/2.0+0.5, wcs->naxisn[1]-ystep/2.0+0.5,
 	     clevel, CPLOT_NSHADES, 1, 0, 0, plfill, 0, distort_map, wcsptr);
     plFree2dGrid(count, nsnap2, nsnap2);
     plcol0(7);
-    plwid(lwid);
+    CPLOT_PLWID(lwid);
     cplot_drawbounds(wcs, wcsout);
     }
 
@@ -1996,8 +2000,8 @@ int	cplot_modchi2(fieldstruct *field)
 
   plvpor(0.91,0.935,0.115,0.885);
   plwind(0.0,1.0,cmin,cmax);
-  plshades(count, 2, CPLOT_NSHADES, NULL, 0.0, 1.0, cmin,cmax, clevel,
-	   CPLOT_NSHADES, 1, 0, 0, plfill, 1, NULL, NULL);
+  plshades((const PLFLT * const*)count, 2, CPLOT_NSHADES, NULL, 0.0, 1.0,
+	   cmin,cmax, clevel, CPLOT_NSHADES, 1, 0, 0, plfill, 1, NULL, NULL);
   plcol0(15);
   plschr(0.0, 0.5);
   plbox("bc", 0.0, 0, "bnstv", 0.0, 0);
@@ -2023,7 +2027,7 @@ INPUT	Pointer to the field.
 OUTPUT	RETURN_OK if everything went fine, RETURN_ERROR otherwise.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	16/04/2012
+VERSION	19/10/2016
  ***/
 int	cplot_modresi(fieldstruct *field)
   {
@@ -2042,7 +2046,7 @@ int	cplot_modresi(fieldstruct *field)
 		**scount,
 		i,j, e, n,nt, naxis, nsnap,nsnap2, flag;
 
-  if (cplot_init(field->rcatname, 1,1, CPLOT_MODRESI) == RETURN_ERROR)
+  if (cplot_init(field->rcatname, CPLOT_MODRESI) == RETURN_ERROR)
     {
     cplot_end(CPLOT_MODRESI);
     return RETURN_OK;
@@ -2070,14 +2074,14 @@ int	cplot_modresi(fieldstruct *field)
   ymin = 0.5;
   ymax = 100.5;
   lwid = plotaaflag? ((CPLOT_AAFAC+1)/2) : 1;
-  plwid(lwid);
+  CPLOT_PLWID(lwid);
   plfont(2);
   plcol0(15);
   plenv((PLFLT)xmin, (PLFLT)xmax, (PLFLT)ymin, (PLFLT)ymax, 1, -1);
   sprintf(str, "#uField %.24s: map of residuals", field->rtcatname);
   plschr(0.0, 1.0);
   pllab("","", str);
-  plwid(0);
+  CPLOT_PLWID(0);
   plcol0(7);
   cplot_drawloccoordgrid(wcsout, xmin, xmax, ymin, ymax);
 
@@ -2138,13 +2142,13 @@ int	cplot_modresi(fieldstruct *field)
 
     wcsptr[0] = wcs;
     wcsptr[1] = wcsout;
-    plshades(count, nsnap2, nsnap2, NULL,
+    plshades((const PLFLT * const*)count, nsnap2, nsnap2, NULL,
 	     xstep/2.0+0.5, wcs->naxisn[0]-xstep/2.0+0.5,
              ystep/2.0+0.5, wcs->naxisn[1]-ystep/2.0+0.5,
 	     clevel, CPLOT_NSHADES, 1, 0, 0, plfill, 0, distort_map, wcsptr);
     plFree2dGrid(count, nsnap2, nsnap2);
     plcol0(7);
-    plwid(lwid);
+    CPLOT_PLWID(lwid);
     cplot_drawbounds(wcs, wcsout);
     }
 
@@ -2155,8 +2159,8 @@ int	cplot_modresi(fieldstruct *field)
 
   plvpor(0.91,0.935,0.115,0.885);
   plwind(0.0,1.0,cmin,cmax);
-  plshades(count, 2, CPLOT_NSHADES, NULL, 0.0, 1.0, cmin,cmax, clevel,
-	   CPLOT_NSHADES, 1, 0, 0, plfill, 1, NULL, NULL);
+  plshades((const PLFLT * const*)count, 2, CPLOT_NSHADES, NULL, 0.0, 1.0,
+	   cmin,cmax, clevel, CPLOT_NSHADES, 1, 0, 0, plfill, 1, NULL, NULL);
   plcol0(15);
   plschr(0.0, 0.5);
   plbox("bc", 0.0, 0, "bnstv", 0.0, 0);
@@ -2175,3 +2179,161 @@ int	cplot_modresi(fieldstruct *field)
   }
 
 
+/****** cplot_snrvsfwhm ******************************************************
+PROTO	cplot_snrvsfwhm(fieldstruct *field, setstruct *set)
+PURPOSE	Plot log(SNR) vs FWHM with flag values
+INPUT	Pointer to the field.
+OUTPUT	RETURN_OK if everything went fine, RETURN_ERROR otherwise.
+NOTES	Global preferences are used.
+AUTHOR	E. Bertin (IAP)
+VERSION	19/10/2016
+ ***/
+int	cplot_snrvsfwhm(fieldstruct *field, setstruct *set)
+  {
+   samplestruct	*sample;
+   char		str[80];
+   PLFLT	**histo,**histo_acc,
+		r[2],g[2],b[2],cpoint[2], clevel[CPLOT_NSHADES], x[2],y[2],
+		aspect, minx,miny, maxx,maxy,
+		scalex,scaley, z,zmax,zmax_acc,
+		lfwhm, lsnr, fmin,fmax;
+   PLINT	lwid;
+   int		e,i,n,s, ix,iy, nx,ny, next;
+
+
+  // Define the plot grid according to the page aspect ratio
+  next = field->next;
+  if (cplot_init(field->rcatname, CPLOT_SNRVSFWHM) == RETURN_ERROR)
+    {
+    cplot_end(CPLOT_SNRVSFWHM);
+    return RETURN_OK;
+    }
+
+  ny = ceil(sqrt((double)next)*plotaspect);
+  nx = ceil((double)next / (double)ny);
+  aspect = (PLFLT)nx / (PLFLT)ny * plotaspect;
+  plssub(nx, ny);
+
+  // Plot boundaries (in log10 units)
+  minx = -0.222;
+  miny = 0.477;
+  maxx = 1.778;
+  maxy = 5.477;
+  scalex = CPLOT_SNRVSFWHM_NX / (maxx - minx);
+  scaley = CPLOT_SNRVSFWHM_NY / (maxy - miny);
+  plscmap1n(256);
+  cpoint[0] = 0.0;
+  cpoint[1] = 1.0;
+
+ for (e=0; e<next; e++) {
+    plAlloc2dGrid(&histo, CPLOT_SNRVSFWHM_NX, CPLOT_SNRVSFWHM_NY);
+    plAlloc2dGrid(&histo_acc, CPLOT_SNRVSFWHM_NX, CPLOT_SNRVSFWHM_NY);
+    zmax = zmax_acc = 0.0;
+    sample = set->sample;
+    for (n=set->nsample; n--; sample++)
+      {
+      if (sample->extindex != e)
+        continue;
+      lsnr = sample->snr > 0.0? log10f(sample->snr) : -10.0;
+      lfwhm = sample->fwhm > 0.0? log10f(sample->fwhm) : -10.0;
+      ix = (int)((lfwhm - minx)*scalex);
+      iy = (int)((lsnr - miny)*scaley);
+      if (ix>=0 && ix<CPLOT_SNRVSFWHM_NX && iy>=0 && iy<CPLOT_SNRVSFWHM_NY)
+        {
+        z = (histo[ix][iy] += 1.0);
+        if (z>zmax)
+          zmax = z;
+        if (!sample->badflag)
+          {
+          z = (histo_acc[ix][iy] += 1.0);
+          if (z>zmax_acc)
+            zmax_acc = z;
+          }
+        }
+      }
+    lwid = plotaaflag? ((CPLOT_AAFAC+1)/2) : 1;
+    CPLOT_PLWID(lwid);
+
+    plscolbg(255,255,255);	/* Force the background colour to white */
+    plscol0(15, 0,0,0);		/* Force the foreground colour to black */
+    plschr(0.0,1.01);
+    plcol0(15);
+    pladv(0);
+    if (next>1) {
+      if (aspect < 1.0)
+        plvpor(0.65 - 0.3 * aspect, 0.65 + 0.3 * aspect, 0.25, 0.85);
+      else
+        plvpor(0.35, 0.95, 0.55 - 0.3 / aspect, 0.55 + 0.3 / aspect);
+    } else
+      plvasp(1.0);
+    plwind((PLFLT)minx, (PLFLT)maxx, (PLFLT)miny, (PLFLT)maxy);
+
+    // Use a non-linear shade level distribution
+    if (zmax>=1.0) {
+      for (i=0; i<CPLOT_NSHADES; i++)
+        clevel[i] = pow(i/(CPLOT_NSHADES-1.0),1.8)*zmax+0.5;
+      r[0] = 1.0; g[0] = 0.8; b[0] = 0.8;
+      r[1] = 0.6; g[1] = 0.1; b[1] = 0.1;
+      plscmap1l(1, 2, cpoint, r, g, b, NULL);
+      plshades((const PLFLT **)histo, CPLOT_SNRVSFWHM_NX, CPLOT_SNRVSFWHM_NY,
+	NULL, minx,maxx, miny,maxy,
+	clevel, CPLOT_NSHADES, 1, 0, 0, plfill, 1, NULL, NULL);
+    } else {
+      plcol0(1);
+      plptex(0.0, maxx/2.0, 1.0, 0.0, 0.5, "No source!");
+    }
+    if (zmax_acc>=1.0) {
+      r[0] = 0.0; g[0] = 0.0; b[0] = 0.0;
+      r[1] = 0.8; g[1] = 0.8; b[1] = 0.8;
+      plscmap1l(1, 2, cpoint, r, g, b, NULL);
+      plimage((const PLFLT **)histo_acc, CPLOT_SNRVSFWHM_NX, CPLOT_SNRVSFWHM_NY,
+	minx,maxx, miny,maxy,
+	0.5, zmax_acc,
+	minx,maxx, miny,maxy);
+    }
+
+  // Draw FWHM selection domain
+    y[0] = (PLFLT)(prefs.minsn > 0.0?
+	log10(prefs.minsn) : -10.0);
+    y[1] = (PLFLT)10.0;
+    x[0] = x[1] = fmin = (PLFLT)(prefs.fwhmrange[0] > 0.0?
+	log10(prefs.fwhmrange[0]) : -10.0);
+    plcol0(15);
+    pllsty(2);
+    plline(2,x,y);
+    x[0] = x[1] = fmax = (PLFLT)(prefs.fwhmrange[1] > 0.0?
+	log10(prefs.fwhmrange[1]) : -10.0);
+    plline(2,x,y);
+    x[0] = x[1] = (PLFLT)(field->psf && field->psf[e] &&
+	field->psf[e]->fwhm > 0.0? log10(field->psf[e]->fwhm) : -10.0);
+    pllsty(3);
+    plline(2,x,y);
+    x[0] = fmin;
+    x[1] = fmax;
+    y[0] = y[1] = (PLFLT)(prefs.minsn > 0.0?
+	log10(prefs.minsn) : -10.0);
+    pllsty(2);
+    plline(2,x,y);
+
+    pllsty(1);
+    plbox("bcflnst", 0.0, 0.0, "bclnstv", 0.0, 0.0);
+    if (next > 1)
+      sprintf(str, "%s[%d]", field->rtcatname, e+1);
+    else
+      sprintf(str, "FWHM selection for %s", field->rtcatname);
+    pllab( "FWHM [pix]", "SNR", str);
+
+    // Free array of points */
+    plFree2dGrid(histo, CPLOT_SNRVSFWHM_NX, CPLOT_SNRVSFWHM_NY); 
+    plFree2dGrid(histo_acc, CPLOT_SNRVSFWHM_NX, CPLOT_SNRVSFWHM_NY); 
+
+  }
+
+  plend();
+  cplot_snrvsfwhm(field, set);	/* Recursive stuff */
+
+  return RETURN_OK;
+  }
+
+
+
