@@ -7,7 +7,7 @@
 *
 *	This file part of:	PSFEx
 *
-*	Copyright:		(C) 1997-2014 Emmanuel Bertin -- IAP/CNRS/UPMC
+*	Copyright:		(C) 1997-2018 IAP/CNRS/SorbonneU
 *
 *	License:		GNU General Public License
 *
@@ -22,7 +22,7 @@
 *	You should have received a copy of the GNU General Public License
 *	along with PSFEx.  If not, see <http://www.gnu.org/licenses/>.
 *
-*	Last modified:		26/02/2014
+*	Last modified:		09/05/2018
 *
 *%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
@@ -61,7 +61,7 @@ INPUT	Array of catalog filenames,
 OUTPUT  Pointer to a set containing samples that match acceptance criteria.
 NOTES   -.
 AUTHOR  E. Bertin (IAP)
-VERSION 19/07/2012
+VERSION 21/09/2015
 */
 setstruct *load_samples(char **filename, int catindex, int ncat, int ext,
 			int next, contextstruct *context)
@@ -264,10 +264,10 @@ setstruct *load_samples(char **filename, int catindex, int ncat, int ext,
     icat = catindex + i;
     if (ext == ALL_EXTENSIONS)
       for (e=0; e<next; e++)
-        set = read_samples(set, filename[icat], fwhmmin[i]/2.0, fwhmmax[i]/2.0,
+        set = read_samples(set, filename[icat], fwhmmin[i], fwhmmax[i],
 			e, next, icat, context, context->pc+i*context->npc);
     else
-      set = read_samples(set, filename[icat], fwhmmin[i]/2.0, fwhmmax[i]/2.0,
+      set = read_samples(set, filename[icat], fwhmmin[i], fwhmmax[i],
 			ext, next, icat, context, context->pc+i*context->npc);
     if (fwhmmode[i]<mode)
       mode = fwhmmode[i];
@@ -321,45 +321,53 @@ INPUT   Pointer to an array of FWHMs,
 	pointer to the upper FWHM range (output).
 OUTPUT  FWHM mode.
 NOTES   -.
-AUTHOR  E. Bertin (IAP, Leiden observatory & ESO)
-VERSION 20/03/2008
+AUTHOR  E. Bertin (IAP)
+VERSION 09/05/2018
 */
 static float	compute_fwhmrange(float *fwhm, int nfwhm, float maxvar,
 		float minin, float maxin, float *minout, float *maxout)
   {
    float	*fwhmt,*fwhmt2,
-		df, dfmin,fmin;
+		df, dfmin,fmin, var;
    int		i, nw;
 
 /* Sort FWHMs */
-   fqmedian(fwhm, nfwhm);
+  fqmedian(fwhm, nfwhm);
 
 /* Find the mode */
-   nw = nfwhm/4;
-   if (nw<4)
-     nw = 1;
-  dfmin = BIG;
-  fmin = 0.0;
-  fwhmt = fwhm;
-  fwhmt2 = fwhm+nw;
-  for (i=nfwhm-nw; i--; fwhmt++,fwhmt2++)
-    {  
-    if ((df = *fwhmt2 - *fwhmt) < dfmin)
-      {
-      dfmin = df;
-      fmin = (*fwhmt2 + *fwhmt)/2.0;
+  nw = nfwhm/4;
+  if (nw < 1)
+    nw = 1;
+  var = BIG;
+
+  for (nw = nfwhm / 4; var > maxvar / 2 && nw > 0; nw /= 2) {
+    dfmin = BIG;
+    fmin = 0.0;
+    fwhmt = fwhm;
+    fwhmt2 = fwhm+nw;
+    for (i=nfwhm-nw; i--; fwhmt++,fwhmt2++)
+      if ((df = *fwhmt2 - *fwhmt) < dfmin) {
+        dfmin = df;
+        fmin = (*fwhmt2 + *fwhmt)/2.0;
       }
+
+    if (nfwhm<2) {
+      fmin = *fwhm;
+      break;
     }
 
-  if (nfwhm<2)
-    fmin = *fwhm;
+    if (fmin <= 0.0)
+      break;
 
-  dfmin = (float)pow((double)maxvar+1.0, 0.3333333);
-  *minout = dfmin>0.0?fmin/dfmin:0.0;
-  if (*minout<minin)
+    var = dfmin / fmin;
+  }
+
+  dfmin = sqrtf((float)maxvar + 1.0f);
+  *minout = dfmin > 0.0 ? fmin / dfmin : 0.0;
+  if (*minout < minin)
     *minout = minin;
-  *maxout = fmin*dfmin*dfmin;
-  if (*maxout>maxin)
+  *maxout = fmin * dfmin;
+  if (*maxout > maxin)
     *maxout = maxin;
 
   return fmin;
@@ -375,8 +383,8 @@ PROTO	setstruct *read_samples(setstruct *set, char *filename,
 PURPOSE	Read point source data for a given set.
 INPUT	Pointer to the data set,
 	catalogue filename,
-	minimum flux radius,
-	maximum flux radius,
+	minimum FWHM,
+	maximum FWHM,
 	current extension,
 	number of extensions,
 	catalogue index
@@ -385,17 +393,17 @@ INPUT	Pointer to the data set,
 OUTPUT  Pointer to a set containing samples that match acceptance criteria.
 NOTES   -.
 AUTHOR  E. Bertin (IAP)
-VERSION 26/02/2014
+VERSION 20/09/2016
 */
 setstruct *read_samples(setstruct *set, char *filename,
-			float frmin, float frmax,
+			float fwhmmin, float fwhmmax,
 			int ext, int next, int catindex,
 			contextstruct *context, double *pcval)
 
   {
    catstruct		*cat;
    tabstruct		*tab, *keytab;
-   keystruct		*key, *vigkey;
+   keystruct		*key;
    samplestruct		*sample;
    t_type		contexttyp[MAXCONTEXT];
    void			*contextvalp[MAXCONTEXT];
@@ -408,33 +416,30 @@ setstruct *read_samples(setstruct *set, char *filename,
    double		contextval[MAXCONTEXT],
 			*dxm,*dym, *cmin, *cmax, dval;
    float		*xm, *ym, *vignet,*vignett, *flux, *fluxrad, *elong,
-			*snr,
-			backnoise, backnoise2, gain, minsn,maxelong;
+			*snr, *dgeox, *dgeoy,
+			backnoise, backnoise2, gain, minsn, ellip, maxellip;
    static int		ncat;
    int			*lxm,*lym,
 			i,j, n, nsample,nsamplemax,
-			vigw, vigh, vigsize, nobj, nt, ndet,
-			maxbad, maxbadflag, ldflag, ext2, pc, contflag;
+			vigw, vigh, vigsize, nobj, nt, ndet, ngood, dgeoflag,
+			badflag, maxbad, maxbadflag, ldflag, ext2, pc;
    short 		*sxm,*sym;
 
 
   maxbad = prefs.badpix_nmax;
   maxbadflag = prefs.badpix_flag;
-  maxelong = (float)(prefs.maxellip < 1.0?
-	(prefs.maxellip + 1.0)/(1.0 - prefs.maxellip)
-	: 100.0);
+  maxellip = (float)prefs.maxellip;
   minsn = prefs.minsn;
 
 /* If a NULL pointer is provided, we allocate a new set */
-  if (!set)
-    {
+  if (!set) {
     set = init_set(context);
-    nsample = nsamplemax = 0;
+    nsample = nsamplemax = ngood = 0;
     ncat = 1;
-    }
-  else
+  } else {
     nsample = nsamplemax = set->nsample;
-
+    ngood = set->ngood;
+  }
   cmin = cmax = (double *)NULL;	/* To avoid gcc -Wall warnings */
   if (set->ncontext)
     {
@@ -520,7 +525,7 @@ setstruct *read_samples(setstruct *set, char *filename,
   if (!(key = name_to_key(keytab, prefs.center_key[1])))
     {
     sprintf(str, "*Error*: %s parameter not found in catalogue ",
-	prefs.center_key[0]);
+	prefs.center_key[1]);
     error(EXIT_FAILURE, str, filename);
     }
    if (key->ttype == T_DOUBLE)
@@ -543,7 +548,7 @@ setstruct *read_samples(setstruct *set, char *filename,
   if (!(key = name_to_key(keytab, rkeyname)))
     {
     sprintf(str, "*Error*: %s parameter not found in catalogue ",
-	prefs.photflux_rkey);
+	rkeyname);
     error(EXIT_FAILURE, str, filename);
     }
   flux = (float *)key->ptr;
@@ -589,17 +594,17 @@ setstruct *read_samples(setstruct *set, char *filename,
   else
     imaflags = NULL;
 
+/* Load SExtractor VIGNET vector */
   if (!(key = name_to_key(keytab, "VIGNET")))
     error(EXIT_FAILURE,
 	"*Error*: VIGNET parameter not found in catalog ", filename);
   vignet = (float *)key->ptr;
   nobj = key->nobj;
 
-  if (key->naxis != 2)
+  if (key->naxis < 2)
     error(EXIT_FAILURE, "*Error*: VIGNET should be a 2D vector", "");
-  vigkey = key;
-  vigw = *(vigkey->naxisn);
-  vigh = *(vigkey->naxisn+1);
+  vigw = *(key->naxisn);
+  vigh = *(key->naxisn+1);
   vigsize = vigw*vigh;
   if (!set->sample)
     {
@@ -608,6 +613,39 @@ setstruct *read_samples(setstruct *set, char *filename,
     set->nvig = vigw*vigh;
     }
 
+/* Load optional SExtractor VIGNET_DGEOX vector */
+  if (prefs.dgeo_flag && (key = name_to_key(keytab, "VIGNET_DGEOX"))) {
+    dgeox = (float *)key->ptr;
+    nobj = key->nobj;
+    if (key->naxis < 2)
+      error(EXIT_FAILURE, "*Error*: VIGNET_DGEOX should be a 2D vector", "");
+    if (*(key->naxisn) != vigw || *(key->naxisn+1) != vigh)
+      error(EXIT_FAILURE, "*Error*: VIGNET_DGEOX size does not match that of ",
+	"VIGNET");
+  } else
+    dgeox = NULL;
+
+/* Load optional SExtractor VIGNET_DGEOY vector */
+  if (prefs.dgeo_flag && (key = name_to_key(keytab, "VIGNET_DGEOY"))) {
+    dgeoy = (float *)key->ptr;
+    nobj = key->nobj;
+    if (key->naxis < 2)
+      error(EXIT_FAILURE, "*Error*: VIGNET_DGEOY should be a 2D vector", "");
+    if (*(key->naxisn) != vigw || *(key->naxisn+1) != vigh)
+      error(EXIT_FAILURE, "*Error*: VIGNET_DGEOY size does not match that of ",
+	"VIGNET");
+  } else
+    dgeoy = NULL;
+
+  if (dgeox && dgeoy)
+    dgeoflag = 1;
+  else {
+    dgeoflag = 0;
+    if (prefs.dgeo_flag)
+      warning("Both VIGNET_DGEOX and VIGNET_DGEOY must be present in catalog: ",
+	"deactivating differential geometry maps");
+  }
+
 /* Try to load the set of context keys */
   kstr = context->name;
   pc = 0;
@@ -674,45 +712,43 @@ setstruct *read_samples(setstruct *set, char *filename,
       }
 
 /*---- Apply some selection over flags, fluxes... */
-    contflag = 0;
+    badflag = 0;
     if (flags && (*flags&prefs.flag_mask))
       {
-      contflag++;
+      badflag |= BAD_SEXFLAG;
       set->badflags++;
       }
     if (wflags && (*wflags&prefs.wflag_mask))
       {
-      contflag++;
+      badflag |= BAD_WEIGHTFLAG;
       set->badwflags++;
       }
     if (imaflags && (*imaflags&prefs.imaflag_mask))
       {
-      contflag++;
+      badflag |= BAD_IMAFLAG;
       set->badwflags++;
       }
     if (*snr<minsn)
       {
-      contflag++;
+      badflag |= BAD_SNR;
       set->badsn++;
       }
-    if (*fluxrad<frmin)
+    if (2.0**fluxrad<fwhmmin)
       {
-      contflag++;
-      set->badfrmin++;
+      badflag |= BAD_SIZE;
+      set->badfwhmmin++;
       }
-    if (*fluxrad>frmax)
+    if (2.0**fluxrad>fwhmmax)
       {
-      contflag++;
-      set->badfrmax++;
+      badflag |= BAD_SIZE;
+      set->badfwhmmax++;
       }
-    if (elong && *elong>maxelong)
+    if (elong && (ellip=(*elong - 1.0) / (*elong + 1.0)) > maxellip)
       {
-      contflag++;
-      set->badelong++;
+      badflag |= BAD_ELLIP;
+      set->badellip++;
       }
-    if (contflag)
-      continue;
-/*-- ... and check the integrity of the sample */
+/*-- ... and check the integrity of the pixel data */
     j = 0;
     vignett = vignet;
     for (i=vigsize; i--; vignett++)
@@ -720,10 +756,10 @@ setstruct *read_samples(setstruct *set, char *filename,
         j++;
     if (maxbadflag && j > maxbad)
       {
+      badflag |= BAD_PIXEL;
       set->badpix++;
-      continue; 
       }
-    
+   
 /*-- Allocate memory for the first shipment */
     if (!set->sample)
       {
@@ -750,11 +786,12 @@ setstruct *read_samples(setstruct *set, char *filename,
     sample->detindex = ndet;
     sample->extindex = ext;
     sample->catindex = catindex;
-
-/*-- Copy the vignet to the training set */
-    memcpy(sample->vig, vignet, vigsize*sizeof(float));
+    sample->badflag = badflag;
 
     sample->norm = *flux;
+    sample->fwhm = 2.0**fluxrad;
+    sample->ellip = elong? ellip : 0.0;
+    sample->snr = *snr;
     sample->backnoise2 = backnoise2;
     sample->gain = gain;
     if (dxm)
@@ -786,8 +823,18 @@ setstruct *read_samples(setstruct *set, char *filename,
       if (dval>cmax[i])
         cmax[i] = dval;
       }
-    make_weights(set, sample);
-    recenter_sample(sample, set, *fluxrad);
+/*-- Copy the vignet to the training set */
+    if (!badflag) {
+      malloc_samplevig(sample, vigsize, dgeoflag);
+      memcpy(sample->vig, vignet, vigsize*sizeof(float));
+      if (dgeoflag) {
+        memcpy(sample->vigdgeox, dgeox, vigsize*sizeof(float));
+        memcpy(sample->vigdgeoy, dgeoy, vigsize*sizeof(float));
+      }
+      make_weights(set, sample);
+      recenter_sample(sample, set, *fluxrad);
+      ngood++;
+    }
     nsample++;
     }
 
@@ -807,6 +854,7 @@ setstruct *read_samples(setstruct *set, char *filename,
   free_cat(&cat, 1); 
 
   set->nsample = nsample;
+  set->ngood = ngood;
 
 /* Don't waste memory! */
   if (nsample)
@@ -965,7 +1013,7 @@ INPUT   set structure pointer,
 OUTPUT  -.
 NOTES   -.
 AUTHOR  E. Bertin (IAP, Leiden observatory & ESO)
-VERSION 26/03/2008
+VERSION 18/09/2015
 */
 void	malloc_samples(setstruct *set, int nsample)
 
@@ -973,17 +1021,11 @@ void	malloc_samples(setstruct *set, int nsample)
    samplestruct	*sample;
    int		n;
 
-  QMALLOC(set->sample, samplestruct, nsample);
+  QCALLOC(set->sample, samplestruct, nsample);
   sample = set->sample;
   for (n=nsample; n--; sample++)
-    {
-    QMALLOC(sample->vig, float, set->nvig);
-    QMALLOC(sample->vigresi, float, set->nvig);
-    QMALLOC(sample->vigweight, float, set->nvig);
-    QMALLOC(sample->vigchi, float, set->nvig);
     if (set->ncontext)
       QMALLOC(sample->context, double, set->ncontext);
-    }
 
   set->nsamplemax = nsample;
 
@@ -999,7 +1041,7 @@ INPUT   set structure pointer,
 OUTPUT  -.
 NOTES   -.
 AUTHOR  E. Bertin (IAP, Leiden observatory & ESO)
-VERSION 26/03/2008
+VERSION 18/09/2015
 */
 void	realloc_samples(setstruct *set, int nsample)
 
@@ -1018,10 +1060,7 @@ void	realloc_samples(setstruct *set, int nsample)
     sample = set->sample + set->nsamplemax;
     for (n = nsample - set->nsamplemax; n--; sample++)
       {
-      QMALLOC(sample->vig, float, set->nvig);
-      QMALLOC(sample->vigresi, float, set->nvig);
-      QMALLOC(sample->vigchi, float, set->nvig);
-      QMALLOC(sample->vigweight, float, set->nvig);
+      memset(sample, 0, sizeof(samplestruct));
       if (set->ncontext)
         QMALLOC(sample->context, double, set->ncontext);
       }
@@ -1031,10 +1070,7 @@ void	realloc_samples(setstruct *set, int nsample)
     sample = set->sample + nsample;
     for (n = set->nsamplemax - nsample; n--; sample++)
       {
-      free(sample->vig);
-      free(sample->vigresi);
-      free(sample->vigchi);
-      free(sample->vigweight);
+      free_samplevig(sample);
       if (set->ncontext)
         free(sample->context);
       }
@@ -1055,7 +1091,7 @@ INPUT   set structure pointer,
 OUTPUT  -.
 NOTES   -.
 AUTHOR  E. Bertin (IAP, Leiden observatory & ESO)
-VERSION 26/03/2008
+VERSION 18/09/2015
 */
 void	free_samples(setstruct *set)
 
@@ -1066,10 +1102,7 @@ void	free_samples(setstruct *set)
   sample = set->sample;
   for (n = set->nsamplemax; n--; sample++)
     {
-    free(sample->vig);
-    free(sample->vigresi);
-    free(sample->vigweight);
-    free(sample->vigchi);
+    free_samplevig(sample);
     if (set->ncontext)
       free(sample->context);
     }
@@ -1082,6 +1115,56 @@ void	free_samples(setstruct *set)
   }
 
 
+/****** malloc_samplevig *******************************************************
+PROTO   void free_samplevig(samplestruct *sample, int npix, int dgeoflag)
+PURPOSE Free pixel memory for a sample.
+INPUT   sample structure pointer,
+        number of pixels,
+	differential geometry map flag.
+OUTPUT  -.
+NOTES   -.
+AUTHOR  E. Bertin (IAP)
+VERSION 15/12/2015
+*/
+void	malloc_samplevig(samplestruct *sample, int npix, int dgeoflag)
+
+  {
+  QMALLOC(sample->vig, float, npix);
+  QMALLOC(sample->vigresi, float, npix);
+  QMALLOC(sample->vigweight, float, npix);
+  QMALLOC(sample->vigchi, float, npix);
+  if (dgeoflag) {
+    QMALLOC(sample->vigdgeox, float, npix);
+    QMALLOC(sample->vigdgeoy, float, npix);
+  }
+
+  return;
+  }
+
+
+/****** free_samplevig *******************************************************
+PROTO   void free_samplevig(samplestruct *sample)
+PURPOSE Free pixel memory for a sample.
+INPUT   sample structure pointer.
+OUTPUT  -.
+NOTES   -.
+AUTHOR  E. Bertin (IAP)
+VERSION 15/12/2015
+*/
+void	free_samplevig(samplestruct *sample)
+
+  {
+  QFREE(sample->vig);
+  QFREE(sample->vigresi);
+  QFREE(sample->vigweight);
+  QFREE(sample->vigchi);
+  QFREE(sample->vigdgeox);
+  QFREE(sample->vigdgeoy);
+
+  return;
+  }
+
+
 /****** remove_sample ********************************************************
 PROTO   samplestruct *remove_sample(setstruct *set, int isample)
 PURPOSE Remove an element from a set of samples.
@@ -1117,6 +1200,53 @@ samplestruct	*remove_sample(setstruct *set, int isample)
   }
 
 
+/****** add_set ************************************************************
+PROTO   void add_set(setstruct *destset, setstruct *set)
+PURPOSE Add the content of one set to another set (without the pixel data).
+INPUT   destination set structure pointer,
+        input set structure pointer.
+OUTPUT  -.
+NOTES   -.
+AUTHOR  E. Bertin (IAP)
+VERSION 20/09/2016
+*/
+void	add_set(setstruct *destset, setstruct *set) {
+   samplestruct	*destsample, *sample;
+   int		c,n, ncontext;
+
+  // (Re-)allocate memory
+  if (!destset->nsamplemax) {
+    destset->nsample = set->nsample;
+    malloc_samples(destset, destset->nsample);
+  } else if ((destset->nsample += set->nsample) > destset->nsamplemax)
+    realloc_samples(destset, destset->nsample);
+  // Restrict to the smallest amount of contexts to avoid issues
+  ncontext = destset->ncontext > set->ncontext? set->ncontext:destset->ncontext;
+  // Set context if necessary
+  if (ncontext && destset->contextscale[0] == 0.0)
+    for (c=0; c<ncontext; c++) {
+      destset->contextoffset[c] = set->contextoffset[c];
+      destset->contextscale[c] = set->contextscale[c];
+      strcpy(destset->contextname[c], set->contextname[c]);
+    }
+  destset->ngood += set->ngood;
+  destsample = destset->sample + destset->nsample - set->nsample;
+  sample = set->sample;
+  for (n=set->nsample; n--; sample++, destsample++) {
+    // Copy scalars
+    *destsample = *sample;
+    // Copy contexts
+    if (ncontext)
+      QMEMCPY(sample->context, destsample->context, double, ncontext);
+    // Don't copy pixel data
+    destsample->vig = destsample->vigresi = destsample->vigweight = destsample->vigchi = NULL;
+  }
+
+  return;
+}
+
+
+
 /****** init_set ************************************************************
 PROTO   setstruct *init_set()
 PURPOSE Allocate and initialize a set structure.
@@ -1124,7 +1254,7 @@ INPUT   -.
 OUTPUT  -.
 NOTES   See prefs.h.
 AUTHOR  E. Bertin (IAP, Leiden observatory & ESO)
-VERSION 26/03/2008
+VERSION 20/09/2016
 */
 setstruct	*init_set(contextstruct *context)
 
@@ -1133,17 +1263,16 @@ setstruct	*init_set(contextstruct *context)
    int		i;
 
   QCALLOC(set, setstruct, 1);
-  set->nsample = set->nsamplemax = 0;
   set->vigdim = 2;
   QMALLOC(set->vigsize, int, set->vigdim);
   set->ncontext = context->ncontext;
   if (set->ncontext)
     {
-    QMALLOC(set->contextoffset, double, set->ncontext);
-    QMALLOC(set->contextscale, double, set->ncontext);
-    QMALLOC(set->contextname, char *, set->ncontext);
+    QCALLOC(set->contextoffset, double, set->ncontext);
+    QCALLOC(set->contextscale, double, set->ncontext);
+    QCALLOC(set->contextname, char *, set->ncontext);
     for (i=0; i<set->ncontext; i++)
-      QMALLOC(set->contextname[i], char, 80);
+      QCALLOC(set->contextname[i], char, 80);
     }
 
   return set;
